package io.github.null2264.tsukumogami.core.commands

import io.github.null2264.tsukumogami.core.Context
import io.github.null2264.tsukumogami.core.annotation.TsukumogamiInternalApi

@OptIn(TsukumogamiInternalApi::class)
class Group(
    name: String,
    alias: Set<String>,
    description: String,
) :
    IGroup,
    Command(
        name,
        alias,
        description,
        ::EmptyArguments,
        { _, _ -> /* No handler for group to match the behaviour of Discord's slash command */ },
    ) {

    override val allCommands: MutableMap<String, Command> = mutableMapOf()

    override fun _addCommand(command: Command) {
        command.parent = this
        super._addCommand(command)
    }

    override suspend fun invoke(context: Context) {
        val subcommandName = context.message.content
            .substringAfter("${context.prefix}${context.invokedWith}")
            .trim()
            .substringBefore(' ')
        val command = allCommands[subcommandName] ?: return
        context.invokedWith += " $subcommandName"
        context.command = command
        command.invoke(context)
    }
}
