package io.github.null2264.tsukumogami.core.koin

import org.koin.core.Koin
import org.koin.core.KoinApplication
import org.koin.core.context.KoinContext
import org.koin.core.error.KoinApplicationAlreadyStartedException
import org.koin.core.module.Module
import org.koin.dsl.KoinAppDeclaration

/**
 * The [KoinContext] for Tsukumogami.
 *
 * To use this context, implement [TsukumogamiKoinComponent].
 *
 * @see org.koin.core.context.GlobalContext
 */
object TsukumogamiKoinContext : KoinContext {
    /** The current [Koin] instance. */
    private var koin: Koin? = null

    /** The current [KoinApplication]. */
    private var koinApp: KoinApplication? = null

    override fun get(): Koin = koin ?: error("KoinApplication has not been started")
    override fun getOrNull(): Koin? = koin
    public fun getKoinApplicationOrNull(): KoinApplication? = koinApp

    private fun register(koinApplication: KoinApplication) {
        if (koin != null) {
            throw KoinApplicationAlreadyStartedException("KoinApplication has already been started")
        }

        koinApp = koinApplication
        koin = koinApplication.koin
    }

    override fun loadKoinModules(module: Module, createEagerInstances: Boolean) {
        get().loadModules(listOf(module), createEagerInstances = createEagerInstances)
    }

    override fun loadKoinModules(modules: List<Module>, createEagerInstances: Boolean) {
        get().loadModules(modules, createEagerInstances = createEagerInstances)
    }

    override fun startKoin(koinApplication: KoinApplication): KoinApplication = synchronized(this) {
        register(koinApplication)
        koinApplication.createEagerInstances()
        koinApplication.allowOverride(true)

        return koinApplication
    }

    override fun startKoin(appDeclaration: KoinAppDeclaration): KoinApplication  = synchronized(this) {
        val koinApplication = KoinApplication.init()

        register(koinApplication)
        appDeclaration(koinApplication)
        koinApplication.createEagerInstances()

        return koinApplication
    }

    override fun stopKoin() {
        koin?.close()
        koin = null
    }

    override fun unloadKoinModules(module: Module) {
        get().unloadModules(listOf(module))
    }

    override fun unloadKoinModules(modules: List<Module>) {
        get().unloadModules(modules)
    }
}
